/* Title:        ActionAdapter,java
   Author:       Markku Lehtinen
   Date:         Nov -97
   Description:  Yleisadapteri eri tyyppisten viestien ksittelemiseen

   Konstruktorin parametrit:

   int action       Tapahtuma, joka aiheutti ActionAdapterin kutsumisen.
                    esim. keyEvent.KEY_PRESSED
                          mouseEvent.MOUSE_CLICKED
                          mouseEvent.MOUSE_PRESSED

   Object parent    Kutsuvan luokan osoite. (konstruktorin kutsussa this).

   String method    Kutsuvaan luokkaan kuuluva metodi, joka
                    kynnistetn, kun action -niminen tapahtuma esiintyi.
                    Parametrina on metodin nimi String -muodossa.
                    Metodilla tytyy olla kaksi Object -tyyppist parametria.

                    Esim.

                      public void button_actionPerformed(Object o, Object event)

                    vlitetn vain metodin nimi: "button_actionPerformed"

                    Object o tarkoittaa objektia, joka on yhteydess
                             viestin ksittelyyn. Muutetaan tyyppimuunnoksella
                             halutun tyyppiseksi luokaksi.

                    Object event Tapahtuma, joka aiheutti ksittelijn kutsun.
                                 Muutetaan tyyppimuunnoksella halutunlaiseksi.
                                 Esim. jos ksittelij on hiiren toimintaa
                                 tutkiva metodi, muutetaan event tyypiksi
                                 MouseEvent. event sislt tiedot tapahtuneesta
                                 toiminnasta.


   Object actionObject

                    Objecti, joka vlitetn kynnistettvlle metodille, joka
                    ksittelee tapahtuman. Edellisesn kohdan Object o.

   ActionAdapter -luokka annetaan parametrina eri Event -tyyppien
   ksittely komponenttiin lisvlle funktiolle addXXXListener.

*/

package omat.adapter;

import java.lang.reflect.*;
import java.lang.*;
import java.awt.event.*;


public class ActionAdapter implements
  java.awt.event.KeyListener,
  java.awt.event.MouseListener,
  java.awt.event.ActionListener
  /* Vain nm toteutettu tll hetkell */
{
  Method method;
  int action;
  Object parent;
  Object[] params = new Object[2];

  public ActionAdapter(int action, Object parent, String method, Object action_object) {
    this.action = action;  this.parent = parent; this.params[0] = action_object;
    this.method = getThisActionMethod(parent,method);
  }

  /* KeyEvents */
  public void keyPressed(KeyEvent e) { if( action == e.KEY_PRESSED ) invoke(e); }
  public void keyReleased(KeyEvent e) { if( action == e.KEY_RELEASED ) invoke(e); }
  public void keyTyped(KeyEvent e) { if( action == e.KEY_TYPED ) invoke(e); }

  /* MouseEvents */
  public void mouseExited(MouseEvent e) { if( action == e.MOUSE_EXITED ) invoke(e); }
  public void mouseEntered(MouseEvent e) { if( action == e.MOUSE_ENTERED ) invoke(e); }
  public void mousePressed(MouseEvent e) { if( action == e.MOUSE_PRESSED ) invoke(e); }
  public void mouseReleased(MouseEvent e) { if( action == e.MOUSE_RELEASED ) invoke(e); }
  public void mouseClicked(MouseEvent e) { if( action == e.MOUSE_CLICKED ) invoke(e);}

  /* ActionEvent */
  public void actionPerformed(ActionEvent e) {invoke(e); }

  void invoke(Object event) {
   try {
     params[1]=event;
     method.invoke(parent,params);
   } catch (Exception e) {
     System.out.println("Error: "+e.toString());
   }
  }

  Method getThisActionMethod(Object source_this, String name) {
     int i;
     Class[] params = new Class[2];
     Method method = null;

     try {
      params[0] = params[1] = source_this.getClass().forName("java.lang.Object");
      method = source_this.getClass().getMethod(name,params);
     } catch (Exception e) {
       System.out.println(e.toString());
       System.exit(0);
     }

     return method;
  }
}



